package com.ibm.websphere.webservices.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFactory;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.sdo.impl.EDataObjectImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.ibm.websphere.webservices.soap.IBMSOAPElement;
import com.ibm.websphere.webservices.soap.IBMSOAPFactory;

/**
 * This class takes care of EMF initialization and defines
 * two methods, serialize and deserialize, used to convert
 * between SOAPElement objects and DocumentRoot objects for
 * all SDOs generated from a single schema.
 */
public class EMFSOAPElementUtil
{
  private ResourceFactoryImpl factory;

  /**
   * Constructs a new utility class,
   * with the side effect of initializing EMF.
   * The XMLResourceFactoryImpl object is the factory responsible
   * for constructing the EMF XML resource objects that contain
   * the SDO models. The call to "getExtensionToFactoryMap"
   * registers the factory with the EMF runtime.
   * @param factory The EMF resource factory that will be
   * used to manage the serialization and deserialization
   * of DocumentRoot objects.
   */
  public EMFSOAPElementUtil ( ResourceFactoryImpl factory )
  {
    this.factory = factory;
    Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xml",factory);
  }

  /**
   * Serializes a DocumentRoot object to a SOAPElement.
   * @param document The EMF/SDO document to serialize.
   * @return A SOAPElement representation of the XML.
   * @throws IOException
   * @throws SOAPException
   */
  public SOAPElement serialize ( EDataObjectImpl document )
  throws IOException, SOAPException
  {
    XMLResourceImpl res = (XMLResourceImpl)factory.createResource(URI.createURI("*.xml"));
    res.getContents().add(document);
    res.getDefaultSaveOptions().put(XMLResource.OPTION_DECLARE_XML,Boolean.FALSE);
    res.setEncoding("UTF-8");
    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
    res.save(byteArrayOutputStream,null);
    byte[] buffer = byteArrayOutputStream.toByteArray();
    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(buffer);
    byteArrayOutputStream.close();
    InputSource inputSource = new InputSource(byteArrayInputStream);
    IBMSOAPFactory soapFactory = (IBMSOAPFactory)SOAPFactory.newInstance();
    return soapFactory.createElementFromInputSource(inputSource);
  }

  /**
   * Deserializes a SOAPElement to a DocumentRoot object.
   * @param element The SOAPElement representation of the XML.
   * @return The resulting EMF/SDO document.
   * @throws IOException
   * @throws SAXException
   */
  public EDataObjectImpl deserialize ( SOAPElement element )
  throws IOException, SAXException
  {
    IBMSOAPElement ibmElement = (IBMSOAPElement)element;
    InputSource inputSource = ibmElement.toInputSource(true);
    InputStream inputStream = inputSource.getByteStream();
    XMLResourceImpl res = (XMLResourceImpl)factory.createResource(URI.createURI("*.xml"));
    res.load(inputStream,null);
    EDataObjectImpl document = (EDataObjectImpl)res.getContents().get(0);
    return document;
  }
}
